﻿//

#include <Windows.h>
#include <stdio.h>
#include <process.h>
#include <time.h>
#include "matrix32.h"

#define MULTI_THREAD_SAMPLE

/*=============================================================*/
/* In this example we use the demo UserCode 1234 and the       */
/* LPT-Port 1.                                                 */
/* - Set DNG_Port = 'U', if you like to test the USB dongle    */
/*=============================================================*/
static short DNG_Port = 85;
static long  UserCode = 1234;


/***************************************************************/
/* Demo for Dongle-Functions                                   */
/***************************************************************/

#if defined(MULTI_THREAD_SAMPLE)
HANDLE hMutex = CreateMutex(NULL, FALSE, NULL);
#define THREAD_COUNT 50
#endif

#if defined(MULTI_THREAD_SAMPLE)
unsigned int __stdcall Matrix_Test(void* pParam)
#else
void Matrix_Test(void* pParam)
#endif
{
	long  DataIn[256];    /* Buffer to read the Dongle data       */
	long  DataOut[256];   /* Buffer for data to be stored         */
	long  DataBlock[2];   /* Data buffer for Encrypt/Decrypt      */
	short RetCode;        /* Return value                         */
	long  API_Version;    /* Version number of the Matrix-API     */
	long  DNG_Version;    /* Dongle version number                */
	short DNG_LPTADR;     /* Adress of LPT port                   */
	short DNG_Count;      /* Number of Dongles connected          */
	short DNG_Mem;        /* Memory size of Dongle                */
	short DNG_MaxVar;     /* Maximum number of data fields        */
	short i;


	/*----------------------------------------------------*/
	/* Init Matrix-API                                    */
	/*----------------------------------------------------*/
	RetCode = Init_MatrixAPI();
	if (RetCode < 0)
	{
		printf("Init_MatrixAPI failed  %d \n", RetCode);
		delete pParam;
		exit(0);
	}
	int idx = *(int*)pParam;

#if defined(MULTI_THREAD_SAMPLE)
	WaitForSingleObject(hMutex, INFINITE);
#endif

	short keyflag = Dongle_GetKeyFlag(UserCode, 1, 85);
	printf("(%d)KeyFlag %d\n", idx, keyflag);

	if (keyflag == 0)
	{
		printf("KeyFlag 0\n");
		unsigned long key[] = { 34343242,55465665,834843784,49039398 };

		if (Dongle_WriteKey(UserCode, key, 1, 85) > 0)
			printf("key written\n");
		else
			printf("key write error\n");
	}
	
#if defined(MULTI_THREAD_SAMPLE)
	ReleaseMutex(hMutex);
#endif

	/*----------------------------------------------------*/
	/* Read the version of the Matrix-API                 */
	/*----------------------------------------------------*/
	API_Version = GetVersionAPI();
	if (API_Version == 0)
	{
		printf("Cannot read API-Version! \n");
		Release_MatrixAPI();
		delete pParam;
		exit(0);
	}
	else
	{
		printf("(%d)Version of Matrix-API: %d.%d \n", idx, HIWORD(API_Version), LOWORD(API_Version));
	}

	/*----------------------------------------------------*/
	/* Search for number of Dongles at DNG_Port           */
	/*----------------------------------------------------*/
	DNG_Count = Dongle_Count(DNG_Port);
	if (DNG_Count > 0)
	{
		printf("(%d)Matrix-Modules : %d \n", idx, DNG_Count);
	}
	else
	{
		printf("Cannot find Matrix-Modules! \n");
		Release_MatrixAPI();
		delete pParam;
		exit(0);
	}

	/*----------------------------------------------------*/
	/* Read memory size of last Dongle at DNG_Port and    */
	/* calculate the maximum number of data fields        */
	/*----------------------------------------------------*/
	DNG_Mem = Dongle_MemSize(DNG_Count, DNG_Port);
	if (DNG_Mem > 0)
	{
		printf("(%d) MemSize of Matrix-Module %d: %d Bytes \n", idx, DNG_Count, DNG_Mem);
	}
	else
	{
		printf("Cannot read MemSize! \n");
		Release_MatrixAPI();
		delete pParam;
		exit(0);
	}
	DNG_MaxVar = DNG_Mem / 4;   /* Anzahl der Datenfelder */

	/*----------------------------------------------------*/
	/* Read Dongle version from last Dongle at DNG_Port   */
	/*----------------------------------------------------*/
	DNG_Version = Dongle_Version(DNG_Count, DNG_Port);
	if (DNG_Version > 0)
	{
		printf("(%d) Version of Matrix-Module %d : %d.%d \n", idx, DNG_Count, HIWORD(DNG_Version), LOWORD(DNG_Version));
	}
	else
	{
		printf("Cannot read the Version number! \n");
		Release_MatrixAPI();
		delete pParam;
		exit(0);
	}

	/*----------------------------------------------------*/
	/* Read 3 data fields from last Dongle at DNG_Port    */
	/* via UserCode and display.                          */
	/*----------------------------------------------------*/

#if defined(MULTI_THREAD_SAMPLE)
	WaitForSingleObject(hMutex, INFINITE);
#endif
	RetCode = Dongle_ReadData(UserCode, DataIn, 3, DNG_Count, DNG_Port);
	if (RetCode < 0)
	{
		printf("Data Read-Error! %d \n", RetCode);
		Release_MatrixAPI();
		delete pParam;
#if defined(MULTI_THREAD_SAMPLE)
		ReleaseMutex(hMutex);
#endif
		exit(0);
	}

	for (i = 0; i < 3; i++)
	{
		printf(" (%d) Var%04d = %ld\n", idx, i + 1, DataIn[i]);
	}
	printf("\n");

	/*----------------------------------------------------*/
	/* Save 3 data fields in last Dongle at DNG_Port via  */
	/* UserCode. The values 101,102,103 will be saved.    */
	/*----------------------------------------------------*/
	for (i = 0; i < 3; i++)
	{
		DataOut[i] = DataIn[i] + 1;
	}

	RetCode = Dongle_WriteData(UserCode, DataOut, 3, DNG_Count, DNG_Port);
	if (RetCode < 0)
	{
		printf("Data Write-Error! \n");
		Release_MatrixAPI();
		delete pParam;
#if defined(MULTI_THREAD_SAMPLE)
		ReleaseMutex(hMutex);
#endif
		exit(0);
	}

#if defined(MULTI_THREAD_SAMPLE)
	ReleaseMutex(hMutex);
#endif

	/*----------------------------------------------------*/
	/* Encrypt/Decrypt DataBlock over the Dongle          */
	/*----------------------------------------------------*/
	DataBlock[0] = 11223344;  /* Clear Data */
	DataBlock[1] = 55667788;  /* Clear Data */
	printf("Clear Data:          %lu  %lu \n", DataBlock[0], DataBlock[1]);

	/*--- Encrypt ---*/
	RetCode = Dongle_EncryptData(UserCode, (unsigned long*)DataBlock, DNG_Count, DNG_Port);
	if (RetCode < 0)
	{
		printf("Data Encryption-Error! \n");
		Release_MatrixAPI();
		delete pParam;
		exit(0);
	}
	printf("(%d) Encrypted Data:  %lu  %lu \n", idx, DataBlock[0], DataBlock[1]);

	/*--- Decrypt ---*/
	RetCode = Dongle_DecryptData(UserCode, (unsigned long*)DataBlock, DNG_Count, DNG_Port);
	if (RetCode < 0)
	{
		printf("Data Decryption-Error! \n");
		Release_MatrixAPI();
		delete pParam;
		exit(0);
	}
	printf("(%d) Decrypted Data:  %lu  %lu \n", idx, DataBlock[0], DataBlock[1]);

	/*----------------------------------------------------*/
	/* Close Matrix-API                                   */
	/*----------------------------------------------------*/
	Release_MatrixAPI();

	delete pParam;

}


int main(int argc, TCHAR** argv)
{
	printf("started\n");

#if !defined(MULTI_THREAD_SAMPLE)

	int* idx = new int(0);
	Matrix_Test(idx);

#else
	HANDLE hThread[THREAD_COUNT];

	for (int i = 0; i < THREAD_COUNT; i++)
	{
		int* idx = new int(i);
		hThread[i] = (HANDLE)_beginthreadex(0, 0, Matrix_Test, idx, 0, 0);
	}

	for (int i = 0; i < THREAD_COUNT; i++)
	{
		if (hThread[i] != NULL)
			WaitForSingleObject(hThread[i], INFINITE);
	}
#endif


	char c = getchar();

	return 0;
}